<!DOCTYPE html>
<html>
<head>
	<meta charset="utf-8">
	<meta name="viewport" content="width=device-width, initial-scale=1">
	<title>Puzzle</title>
</head>
<body>

	<style>
		.canvas{
			border: solid thin #888;
			width:600px;
			height:600px;
			flex: 1;
		}

		.canvas-holder{
			margin: auto;
			margin-top:50px;
			width:100%;
			max-width:900px;
			background-color: #eee;
			display: flex;
		}
		.cropped-images{
			flex:1;
		}

		.btn{
			padding:10px;
			border: solid thin #888;
			cursor: pointer;
			display: inline-block;
			margin:2px;
		}
	</style>
	<div class="canvas-holder">
		<canvas class="canvas"></canvas>
		<div class="cropped-images">
			<div class="controls">
				<label>
					<input onchange="addImage(this.files[0])" class="image-input" type="file" name="image" style="display:none">
					<div class="btn">Load Image</div>
				</label>
				<div class="btn" onclick="saveSelected()">Save Selected</div>
			</div>
			<div class="images"></div>
		</div>
	</div>

</body>
</html>

<script>
	
	const canvas = document.querySelector(".canvas");
	const ctx = canvas.getContext('2d');

	canvas.width = 600;
	canvas.height = 600;

	const cropped_images_div = document.querySelector(".cropped-images");

	canvas.addEventListener('mousemove',mousemoved);
	canvas.addEventListener('mousedown',mousedown);
	window.addEventListener('mouseup',mouseup);

	var mouseIsDown = false;
	var imageAdded = false;
	var mouseDownPos = {x:0,y:0};
	var tempBoxDownPos = {x:0,y:0};
	var mainImage = null;
	var tempBox = {};
	var drawMode = null;
	var SELECTIONS = [];

	function mousemoved(e)
	{
		if(mouseIsDown){

			let mouseX = (e.clientX - canvas.offsetLeft) + window.scrollX;
			let mouseY = (e.clientY - canvas.offsetTop) + window.scrollY;

			if(!imageAdded)
			{
				alert("Please add an image first!");
				return;
			}

			ctx.clearRect(0,0,canvas.width,canvas.height);
			redraw();
			ctx.strokeStyle = "grey";
			
			if(drawMode == null && (tempBox && ((mouseX > tempBox.x && mouseX < (tempBox.x + tempBox.width)) && (mouseY > tempBox.y && mouseY < (tempBox.y + tempBox.height)))))
			{
				drawMode = false;
			}

			if(drawMode == false)
			{
				
				//move existing box
				tempBox.x = tempBoxDownPos.x + (mouseX - mouseDownPos.x);
				tempBox.y = tempBoxDownPos.y + (mouseY - mouseDownPos.y);

				ctx.strokeRect(tempBox.x,tempBox.y,tempBox.width,tempBox.height);
			}else{

				drawMode = true;
				//draw new box
				let width = mouseX - mouseDownPos.x;
				let height = mouseY - mouseDownPos.y;
				ctx.strokeRect(mouseDownPos.x,mouseDownPos.y,width,height);

				//save temp box params
				tempBox.x = mouseDownPos.x;
				tempBox.y = mouseDownPos.y;
				tempBox.width = width;
				tempBox.height = height;

			}
		}
	}

	function mousedown(e)
	{
		mouseIsDown = true;
		mouseDownPos = {
			x:(e.clientX - canvas.offsetLeft) + window.scrollX,
			y:(e.clientY - canvas.offsetTop) + window.scrollY
		};

		if(tempBoxDownPos.x == 0)
		{
			tempBoxDownPos.x = tempBox.x;
			tempBoxDownPos.y = tempBox.y;
		}

	}

	function mouseup(e)
	{
		mouseIsDown = false;
		tempBoxDownPos.x = 0;
		tempBoxDownPos.y = 0;
		drawMode = null;
	}

	function addImage(file)
	{
		let img = new Image();
		img.src = URL.createObjectURL(file);

		img.onload = function()
		{
			imageAdded = true;
			let height = img.naturalHeight / img.naturalWidth;
			ctx.drawImage(img, 0, 0, canvas.width, height * canvas.width);
			mainImage = img;
		}
	}

	function redraw()
	{
		if(mainImage)
		{
			let height = mainImage.naturalHeight / mainImage.naturalWidth;
			ctx.drawImage(mainImage, 0, 0, canvas.width, height * canvas.width);

			//redraw all boxes
			for (var i = 0; i < SELECTIONS.length; i++) {
				
				ctx.fillStyle = 'white';
				ctx.fillRect(SELECTIONS[i].x,SELECTIONS[i].y,SELECTIONS[i].width,SELECTIONS[i].height);
				ctx.strokeRect(SELECTIONS[i].x,SELECTIONS[i].y,SELECTIONS[i].width,SELECTIONS[i].height);
				
			}
		}
	}

	function saveSelected()
	{
		if(typeof tempBox.x != 'undefined')
		{

			SELECTIONS[SELECTIONS.length] = tempBox;

			//export image
			let canv = document.createElement('canvas');
			let canvCtx = canv.getContext('2d');

			canv.width = tempBox.width;
			canv.height = tempBox.height;

			//copy data
			let data = ctx.getImageData(tempBox.x,tempBox.y,tempBox.width,tempBox.height);
			canvCtx.putImageData(data,0,0);

			//add images to list
			let img = document.createElement('img');
			img.src = canv.toDataURL();

			document.querySelector(".cropped-images .images").appendChild(img);
			tempBox = {};
			redraw();
		}else{
			alert("please make a selection first!");
		}
		
	}

</script>